##
## Example of a spatial point proccess
## Data from the splancs package
## The example tranfer the data to the DBMS
## as well a raster with a kernel estimate of the intensity function
##
## Last change: 20060610 Pedro


## 1. Getting started
require(aRT)
require(splancs)
data(bodmin)

## Kernel estimate from the intensity function of the
## point proccess performed within R using splancs

ker <- kernel2d(as.points(bodmin), bodmin$poly, h0=2, nx=100, ny=100)
## plot by splancs
image(ker, asp=1)

## 2. Transfering data and image (raster) to a DB
con = openConn()
con

if(any(showDbs(con)=="bodmin"))
  deleteDb(con, "bodmin", force=T)


#rm(db) ; gc()
db = createDb(con, "bodmin")

## 2.1 Preparing and transfering the points
xy <- as.data.frame(bodmin[1:2])
coordinates(xy) <- c("x", "y")

##
l_points <- importSpData(db, xy, "coords")

## this avoids (encapsulates):
## converting to SpatialPointsDataFrame (with ID specification).
## creating a layers, addpoints() and creating a table

## 2.2 Preparing and transfering the image (raster)

## converting the kernel to "sp"
g <- cbind(expand.grid(x = ker$x, y = ker$y), as.vector(ker$z))
coordinates(g) <- c("x", "y")
gridded(g) <- TRUE

## plot from "sp"
image(g)
contour(ker, add = T)  
fullgrid(g)=TRUE

l_kernel <- createLayer(db, "kernel")
addRaster(l_kernel,g)

## Ploting from the DB -- plot from aRT
plot(l_kernel)
plot(l_points, add=T)

##
## 3. (Optional) Setting visualisations for terraView
##

thp <- createTheme(l_points, "points", view="view")

th <- createTheme(l_kernel, "raster", view="view")
setVisual(th, visualRaster(color=terrain.colors(7)), mode="r")

## splancs kernel objects are also accepted

l_splancs <- createLayer(db,"kernelsplancs")
addRaster(l_splancs, ker)

th <- createTheme(l_splancs, "kernelsplancs", view="view")
setVisual(th, visualRaster(), mode="r")

